@@ -287,19 +287,14 @@ pub(super) struct RunOpts {
     pub(super) reuse_build: ReuseBuildOpts,
 }
 
-#[derive(Copy, Clone, Debug, ValueEnum)]
+#[derive(Copy, Clone, Debug, ValueEnum, Default)]
 pub(crate) enum PlatformFilterOpts {
     Target,
     Host,
+    #[default]
     Any,
 }
 
-impl Default for PlatformFilterOpts {
-    fn default() -> Self {
-        Self::Any
-    }
-}
-
 impl From<PlatformFilterOpts> for Option<BuildPlatform> {
     fn from(opt: PlatformFilterOpts) -> Self {
         match opt {
@@ -310,20 +305,16 @@ impl From<PlatformFilterOpts> for Option<BuildPlatform> {
     }
 }
 
-#[derive(Copy, Clone, Debug, ValueEnum)]
+#[derive(Copy, Clone, Debug, ValueEnum, Default)]
 pub(super) enum ListType {
+    #[default]
     Full,
     BinariesOnly,
 }
 
-impl Default for ListType {
-    fn default() -> Self {
-        Self::Full
-    }
-}
-
-#[derive(Copy, Clone, Debug, ValueEnum)]
+#[derive(Copy, Clone, Debug, ValueEnum, Default)]
 pub(super) enum MessageFormatOpts {
+    #[default]
     Human,
     Json,
     JsonPretty,
@@ -339,12 +330,6 @@ impl MessageFormatOpts {
     }
 }
 
-impl Default for MessageFormatOpts {
-    fn default() -> Self {
-        Self::Human
-    }
-}
-
 #[derive(Debug, Args)]
 #[command(next_help_heading = "Filter options")]
 pub(super) struct TestBuildFilter {
@@ -708,12 +693,11 @@ impl TestRunnerOpts {
         // Warn on conflicts between options. This is a warning and not an error
         // because these options can be specified via environment variables as
         // well.
-        if self.test_threads.is_some() {
-            if let Some(reasons) =
+        if self.test_threads.is_some()
+            && let Some(reasons) =
                 no_run_no_capture_reasons(self.no_run, cap_strat == CaptureStrategy::None)
-            {
-                warn!("ignoring --test-threads because {reasons}");
-            }
+        {
+            warn!("ignoring --test-threads because {reasons}");
         }
 
         if self.retries.is_some() && self.no_run {
@@ -920,15 +904,15 @@ impl ReporterOpts {
 
         let reasons = no_run_no_capture_reasons(no_run, no_capture);
 
-        if self.failure_output.is_some() {
-            if let Some(reasons) = reasons {
-                warn!("ignoring --failure-output because {}", reasons);
-            }
+        if self.failure_output.is_some()
+            && let Some(reasons) = reasons
+        {
+            warn!("ignoring --failure-output because {}", reasons);
         }
-        if self.success_output.is_some() {
-            if let Some(reasons) = reasons {
-                warn!("ignoring --success-output because {}", reasons);
-            }
+        if self.success_output.is_some()
+            && let Some(reasons) = reasons
+        {
+            warn!("ignoring --success-output because {}", reasons);
         }
         if self.status_level.is_some() && no_run {
             warn!("ignoring --status-level because --no-run is specified");
