@@ -18,6 +18,15 @@ use nextest_metadata::MismatchReason;
 use quick_junit::ReportUuid;
 use std::{collections::BTreeMap, fmt, num::NonZero, process::ExitStatus, time::Duration};
 
+/// A reporter event.
+#[derive(Clone, Debug)]
+pub enum ReporterEvent<'a> {
+    /// A periodic tick.
+    Tick,
+
+    /// A test event.
+    Test(Box<TestEvent<'a>>),
+}
 /// A test event.
 ///
 /// Events are produced by a [`TestRunner`](crate::runner::TestRunner) and
@@ -161,6 +170,22 @@ pub enum TestEventKind<'a> {
         running: usize,
     },
 
+    /// Show test-specific progress.
+    ///
+    /// This event is emitted shortly after a test starts running, and is used
+    /// as a trigger to show test-related progress. This is to prevent many very
+    /// short-lived tests from overwhelming the display code.
+    TestShowProgress {
+        /// If a stress test is being run, the stress index, starting from 0.
+        stress_index: Option<StressIndex>,
+
+        /// Retry data.
+        retry_data: RetryData,
+
+        /// The test instance that was started.
+        test_instance: TestInstance<'a>,
+    },
+
     /// A test was slower than a configured soft timeout.
     TestSlow {
         /// If a stress test is being run, the stress index, starting from 0.
