@@ -743,6 +743,13 @@ impl<'a> ExecutorContext<'a> {
             total_attempts: test.retry_data.total_attempts,
             stress_current: test.stress_index.map(|s| s.current),
             stress_total: test.stress_index.and_then(|s| s.total.map(|t| t.get())),
+            global_slot: test.cx.global_slot(),
+            group_slot: test.cx.group_slot(),
+            test_group: test
+                .settings
+                .test_group()
+                .custom_name()
+                .map(|s| s.to_owned()),
         });
 
         // If assigning the child to the job fails, ignore this. This can happen if the process has
@@ -758,6 +765,11 @@ impl<'a> ExecutorContext<'a> {
         // Use a pausable_sleep rather than an interval here because it's much
         // harder to pause and resume an interval.
         let mut interval_sleep = std::pin::pin!(crate::time::pausable_sleep(slow_timeout.period));
+        // Use an event that happens shortly after the test starts to avoid
+        // overwhelming progress bar display code.
+        let mut show_progress_sleep =
+            std::pin::pin!(crate::time::pausable_sleep(Duration::from_millis(10)));
+        let mut show_progress_sleep_done = false;
 
         let mut timeout_hit = 0;
 
@@ -769,6 +781,10 @@ impl<'a> ExecutorContext<'a> {
         let (res, leak_info) = {
             let res = loop {
                 tokio::select! {
+                    _ = &mut show_progress_sleep, if !show_progress_sleep_done => {
+                        _ = resp_tx.send(test.show_progress_event());
+                        show_progress_sleep_done = true;
+                    }
                     () = child_acc.fill_buf(), if !child_acc.fds.is_done() => {}
                     res = child.wait() => {
                         // The test finished executing.
@@ -1003,10 +1019,10 @@ impl BackoffIter {
                 let exp_delay = delay.mul_f64(factor);
 
                 // Stop multiplying the exponential factor if delay is greater than max_delay.
-                if let Some(max_delay) = max_delay {
-                    if exp_delay > max_delay {
-                        return (max_delay, jitter);
-                    }
+                if let Some(max_delay) = max_delay
+                    && exp_delay > max_delay
+                {
+                    return (max_delay, jitter);
                 }
 
                 let next_factor = self.current_factor * Self::BACKOFF_EXPONENT;
@@ -1094,6 +1110,14 @@ pub(super) struct TestPacket<'a> {
 }
 
 impl<'a> TestPacket<'a> {
+    fn show_progress_event(&self) -> ExecutorEvent<'a> {
+        ExecutorEvent::ShowProgress {
+            stress_index: self.stress_index,
+            retry_data: self.retry_data,
+            test_instance: self.test_instance,
+        }
+    }
+
     fn slow_event(&self, elapsed: Duration, will_terminate: Option<Duration>) -> ExecutorEvent<'a> {
         ExecutorEvent::Slow {
             stress_index: self.stress_index,
